/**
*	@author: Thomas Brandého
*/

#include "input.h"


#include <iostream>
#include <SFML/Window.hpp>
#include <list>

Input::Input(sf::Window *w) :
    m_window(w),
    m_closeRequested(false)
{
    m_heldkeys = std::vector<sf::Keyboard::Key>();
    m_actions = std::vector<int>();
    m_window->setKeyRepeatEnabled(false);
}

void Input::setKeysMap(IKeysMap km){
    m_keysmap = km;
}

void Input::updateEvents(){
	sf::Event event;
	KeyBindings kb;
	char c;

	/* reset variables */
    m_closeRequested = false;
    m_hasBeenResized = false;
    m_delta_vertical_scroll = 0;

	/* global affectation */
    kb = m_keybindings[m_current_context];
    m_actions.clear();

	/* event-parsing loop */
    while(m_window->pollEvent(event)){
//        std::cout << event.type << std::endl;
		switch(event.type){
			case sf::Event::Closed:
                m_closeRequested = true;
				break;
			case sf::Event::TextEntered:
				c = (char) event.text.unicode;
                m_buffer.append(&c);
				break;
			case sf::Event::KeyPressed:
                m_actions.push_back(kb.getPressedAction(event.key.code));
                m_heldkeys.push_back(event.key.code);
				break;
			case sf::Event::KeyReleased:
                m_actions.push_back(kb.getReleasedAction(event.key.code));
				releaseHeldKeys(event.key.code);
				break;
			case sf::Event::MouseButtonPressed:
                m_actions.push_back(kb.getPressedAction(sf::Keyboard::KeyCount + event.mouseButton.button));
                m_heldkeys.push_back((sf::Keyboard::Key) (sf::Keyboard::KeyCount + event.mouseButton.button));
				break;
			case sf::Event::MouseButtonReleased:
                m_actions.push_back(kb.getReleasedAction(sf::Keyboard::KeyCount + event.mouseButton.button));
				releaseHeldKeys((sf::Keyboard::Key) (sf::Keyboard::KeyCount + event.mouseButton.button));
				break;
			case sf::Event::MouseWheelScrolled:
				if (event.mouseWheelScroll.wheel == sf::Mouse::VerticalWheel)
                    m_delta_vertical_scroll = event.mouseWheelScroll.delta;
				break;
			case sf::Event::MouseMoved:
                m_last_mouse_position = m_mouse_position;
                m_mouse_position = sf::Mouse::getPosition();
				break;
			case sf::Event::MouseEntered:
				// action MouseEntered
				break;
            case sf::Event::MouseLeft:
                //action MouseLeft
                break;
            case sf::Event::Resized:
                m_hasBeenResized = true;
                break;
        }
	}
    for (auto key: m_heldkeys){
        m_actions.push_back(kb.getHoldAction(key));
	}
}


std::vector<int> Input::getActions()
{
    return m_actions;
}


/* context-related functions */
void Input::addContext(Context context)
{
    m_contexts.push_back(context);
}

void Input::setCurrentContext(std::string context_name){
    m_current_context = context_name;
}

void Input::updateKeyBindings(){
    m_keybindings.clear();
    for (auto iter= m_contexts.begin(); iter != m_contexts.end(); ++iter)
        m_keybindings[iter->getName()]= KeyBindings(*iter,m_keysmap);
}

/* window-related function */

bool Input::isCloseRequested() const
{
    return m_closeRequested;
}

bool Input::isResized() const
{
    return m_hasBeenResized;
}

/* keyboard-related functions */

bool Input::isKeyPressed(int key) const
{
	return sf::Keyboard::isKeyPressed((sf::Keyboard::Key) key);
}

void Input::releaseHeldKeys(sf::Keyboard::Key keycode){
    auto iter = m_heldkeys.begin();
	while(*iter != keycode) ++iter;
    m_heldkeys.erase(iter);
}

/* mouse-related functions */

sf::Vector2i Input::getPosition() const
{
    return m_mouse_position;
}

sf::Vector2i Input::getDeltaPosition() const
{
    return m_mouse_position - m_last_mouse_position;
}

float Input::getDeltaVerticalScroll() const
{
    return m_delta_vertical_scroll;
}

/* ------ */

void Input::test()
{
    KeyBindings	kb = m_keybindings[m_current_context];
	int action;
	action = kb.getPressedAction(sf::Keyboard::I);
	if (action != NO_ACTION)
		std::cerr << action << std::endl;
	action = kb.getPressedAction(sf::Keyboard::O);
	if (action != NO_ACTION)
		std::cerr << action << std::endl;
}
